package com.ruoyi.project.check.mapper;

import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.common.annotation.DataScopePlant;
import com.ruoyi.project.base.domain.TBasePoint;
import com.ruoyi.project.check.domain.TCheckCheckpoints;

/**
 * 检测点Mapper接口
 *
 * @author ruoyi
 * @date 2022-12-01
 */
public interface TCheckCheckpointsMapper  extends BaseMapper<TCheckCheckpoints>
{
    /**
     * 查询检测点
     *
     * @param checkId 检测点主键
     * @return 检测点
     */
    public TCheckCheckpoints selectTCheckCheckpointsByCheckId(Long checkId);

    /**
     * 查询检测点列表
     *
     * @param tCheckCheckpoints 检测点
     * @return 检测点集合
     */
    @DataScopePlant(deptAlias = "d")
    public List<TCheckCheckpoints> selectTCheckCheckpointsList(TCheckCheckpoints tCheckCheckpoints);
    public TCheckCheckpoints selectTCheckCheckpointByEntity(TCheckCheckpoints tCheckCheckpoints);
    public List<TCheckCheckpoints> selectPointsByIds(Long[] ids);

    /**
     * 新增检测点
     *
     * @param tCheckCheckpoints 检测点
     * @return 结果
     */
    public int insertTCheckCheckpoints(TCheckCheckpoints tCheckCheckpoints);
    public int insertTCheckCheckpointsByList(List<TCheckCheckpoints> tCheckCheckpoints);

    /**
     * 修改检测点
     *
     * @param tCheckCheckpoints 检测点
     * @return 结果
     */
    public int updateTCheckCheckpoints(TCheckCheckpoints tCheckCheckpoints);
    public int updateTCheckCheckpointsByCheckIds(TCheckCheckpoints tCheckCheckpoints);
    public int updateTCheckCheckpointsByUpload(TCheckCheckpoints tCheckCheckpoints);
    public int updateApproveStatusByIds(TCheckCheckpoints tCheckCheckpoints);
    public int deleteCheckpoints(TCheckCheckpoints tCheckCheckpoints);

    /**
     * 删除检测点
     *
     * @param checkId 检测点主键
     * @return 结果
     */
    public int deleteTCheckCheckpointsByCheckId(Long checkId);

    /**
     * 批量删除检测点
     *
     * @param checkIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTCheckCheckpointsByCheckIds(Long[] checkIds);
}
